from typing import Optional
import sys
import click
from rich.console import Console

from inni.config import default_config, read_config
from inni.loader import load_module

console = Console()
error_console = Console(stderr=True, style="red bold")


@click.group()
@click.option(
    "-c", "--config", type=click.Path(exists=True, dir_okay=False, readable=True)
)
@click.pass_context
def inni(ctx: click.Context, config: Optional[str]):
    ctx.ensure_object(dict)
    if config is None:
        config = str(default_config())
    ctx.obj["config"] = read_config(config)


def _login_out(ctx: click.Context, login: bool):
    config = ctx.obj["config"]
    module_names = config["inni"]["login" if login else "logout"]
    prompts = config["inni"]["prompts"]
    modules = {}

    for module_name in module_names:
        try:
            module = load_module(module_name)
            module_config = config["modules"].get(module_name, {})
            modules[module_name] = module(module_config)
        except ModuleNotFoundError as err:
            error_console.print(err.msg)
            sys.exit(0)

    variables_to_prompt = []
    for module in modules.values():
        variables_to_prompt += module.template_variables()["login" if login else "logout"]

    responses = {}
    for var in variables_to_prompt:
        prompt = prompts.get(var, f"{var}:").strip() + " "
        responses[var] = console.input("[blue]" + prompt)

    console.print(f"\n[green bold]Logging {'In' if login else 'Out'}\n")
    for name, module in modules.items():
        console.rule(f"[bold cyan]Module: {name}", align="left")
        if login:
            module.login(responses)
        else:
            module.logout(responses)

@inni.command()
@click.pass_context
def login(ctx: click.Context):
    return _login_out(ctx, True)


@inni.command()
@click.pass_context
def logout(ctx: click.Context):
    return _login_out(ctx, False)
